require 'faraday'

module QueryInterface
  module Client
    class API
      attr_accessor :connection, :url, :options 

      def self.setup(options={}, &faraday_block)
        @default_api = self.new(options)
        @default_api.setup_connection(&faraday_block)
      end

      def initialize(options)
        if options.include?(:url)
          self.url = options.delete(:url)
        end
        self.options = options
      end

      def setup_connection(&faraday_block)
        Faraday.new do |connection|
          self.connection = connection
          yield connection if block_given?
        end
        self
      end

      def request(options={})
        method = options.delete(:_method)
        path = options.delete(:_path)
        headers = options.delete(:_headers)
        response = self.connection.send method do |request|
          request.headers.merge!(headers) if headers
          if method == :get
            # For GET requests, treat additional parameters as querystring data
            request.url path, options
          else
            # For POST, PUT and DELETE requests, treat additional parameters as request body
            request.url path
            request.body = options
          end
        end
        {parsed_data: response.env[:body], response: response}
      end

    end
  end
end
