require 'faraday'

module QueryInterface
  module Client
    class JsonMiddleware < Faraday::Response::Middleware

      def parse_json(body)
        body = '{}' if body.blank?
        json = JSON.parse(body, symbolize_names: true)
        unless json.is_a?(Hash) || json.is_a?(Array)
          raise "Response must be Hash or Array, got: #{body.inspect}"
        end
        json
      end

      def parse(body)
        json = parse_json(body)
        errors = json.delete(:errors) || {}
        metadata = json.delete(:metadata) || {}
        {
          :data => json,
          :errors => errors,
          :metadata => metadata
        }
      end

      def on_complete(env)
        env[:body] = case env[:status]
        when 204
          parse('{}')
        else
          parse(env[:body])
        end
      end
    end
  end
end
