module QueryInterface
  module Client
    module Request
      def self.included(base)
        base.extend(ClassMethods)
        base.include(InstanceMethods)
        base.instance_variable_set(:@_qi_collection_path_fragments, nil)
        base.instance_variable_set(:@_qi_resource_path_fragments, nil)
      end

      module ClassMethods
        def resource_name
          self.name.underscore.pluralize
        end

        def collection_path_fragments(*fragments)
          unless fragments.empty?
            @_qi_collection_path_fragments = [self.default_api.url] + fragments
          end
          @_qi_collection_path_fragments || [self.default_api.url, self.resource_name]
        end

        def resource_path_fragments(*fragments)
          unless fragments.empty?
            @_qi_resource_path_fragments = [self.default_api.url] + fragments
          end
          @_qi_resource_path_fragments || self.collection_path_fragments + [:id]
        end

        def default_api
          QueryInterface::Client::API.instance_variable_get(:@default_api)
        end

        def collection_path(path=nil)
          fragments = self.collection_path_fragments.dup
          fragments << path unless path.blank?
          fragments.join("/")
        end

        # TODO: Discern by "am i an instance or a class" what collection and member routes do
        def member_path(path=nil, keys={})
          fragments = self.resource_path_fragments.map do |fragment|
            if fragment.is_a?(Symbol)
              if keys.has_key?(fragment)
                keys[fragment]
              else
                nil
              end
            else
              fragment
            end
          end.compact
          fragments << path unless path.blank?
          fragments.join("/")
        end

        def raw_request(method, params, keys, path)
          if keys
            params[:_path] = self.member_path(path, keys)
          else
            params[:_path] = self.collection_path(path)
          end
          params[:_method] = method
          if params.include?(:_headers)
            params[:_headers] = params[:_headers].merge(self.default_api.options[:headers])
          else
            params[:_headers] = self.default_api.options[:headers]
          end
          result = self.default_api.request(params)
          {parsed_data: result[:parsed_data], response: result[:response], errors: result[:errors]}
        end

        [:get, :post, :put, :delete].each do |method|
          define_method "#{method}_raw" do |options={}|
            params = options.delete(:params) { {} }
            keys = options.delete(:keys) { {} }
            path = options.delete(:path)
            self.raw_request(method, params, keys, path)
          end
        end
      end

      module InstanceMethods
        def keys
          Hash[self.class.primary_keys.map {|primary_key| [primary_key, self.send(primary_key)]}]
        end

        [:get, :post, :put, :delete, :patch].each do |method|
          define_method "#{method}_raw" do |options={}|
            params = options.delete(:params) { {} }
            path = options.delete(:path)
            self.class.raw_request(method, params, self.keys, path)
          end
        end
      end
    end
  end
end
