module QueryInterface
  module Client
    class Association
      attr_accessor :basemodel, :klass, :autoresolve, :dataset

      def initialize(basemodel, name, options={})
        self.basemodel = basemodel
        self.klass = (options.delete(:class_name) {name.to_s.singularize.camelize}).constantize
        self.autoresolve = options.delete(:autoresolve) {true}
        self.dataset = options.delete(:dataset) {name}
      end

      def query(id)
        self.basemodel.query.instance(id).context(self.dataset, self.klass)
      end
    end

    module Resource
      def self.included(base)
        base.instance_variable_set(:@associations, {})
        base.extend(ClassMethods)
        base.include(InstanceMethods)
      end

      module ClassMethods
        def first(*args)
          self.query.order('id').first
        end

        def last(*args)
          self.query.order('id').last
        end

        def query
          LazyQuery.new(self)
        end

        def association(name, options={})
          association = QueryInterface::Client::Association.new(self, name, options)
          define_method name do
            association.query(self.id)
          end
          @associations[name] = association
        end

        def [](id)
          self.query.instance(id).evaluate
        end

        def all
          self.query.evaluate
        end
      end

      module InstanceMethods
        def associations
          self.class.instance_variable_get(:@associations)
        end

        def associations_resolved?
          self.associations.map do |name, association|
            [name, (association.autoresolve || self.send(name).count == 0)]
          end.to_h
        end

        def has_unresolved_dependencies?
          !self.associations_resolved?.values.all?
        end
      end
    end
  end
end
