module QueryInterface
  module Client
    class Conversion
      attr_accessor :set_block, :get_block

      def initialize
        self.get_block = self.set_block = Proc.new { |value| value }
      end

      def set(&set_block)
        self.set_block = set_block
      end

      def get(&get_block)
        self.get_block = get_block
      end

      def apply_set(value)
        self.set_block.call(value)
      end

      def apply_get(value)
        self.get_block.call(value)
      end
    end

    def self.types
      @_types ||= {}
    end

    def self.register_type(type, &registry_block)
      conversion = Conversion.new
      if block_given?
        conversion.instance_eval &registry_block
      end
      self.types[type] = conversion
    end

    register_type :string do
      set { |value| value.to_s }
    end

    register_type :integer do
      set do |value|
        value.blank? ? nil : value.to_i
      end
    end

    register_type :float do
      set do |value|
        value.blank? ? nil : value.to_f
      end
    end

    register_type :ip do
      set { |value| IPAddr.new(value) }
      get { |value| value.to_s }
    end

    register_type :date_time do
      set { |value| value.to_datetime }
      get { |value| value.to_s }
    end

    register_type :date do
      set { |value| value.to_date }
      get { |value| value.to_s }
    end

    register_type :mac do
      set { |value| NetAddr::EUI.create(value) }
      get { |value| value.address(Delimiter: ':') }
    end

    register_type :boolean do
      set do |value|
        if [true, 1, '1', 't', 'T', 'true', 'TRUE', 'on'].include?(value)
          true
        else
          false
        end
      end
    end

    register_type :raw
  end
end
